"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const url_1 = require("url");
const node_1 = require("servie/dist/node");
const servie_errorhandler_1 = require("servie-errorhandler");
const servie_finalhandler_1 = require("servie-finalhandler");
/**
 * Extends HTTP requests with AWS Lambda context.
 */
class LambdaRequest extends node_1.Request {
    constructor(input, options) {
        super(input, options);
        this.context = options.context;
        this.event = options.event;
    }
}
exports.LambdaRequest = LambdaRequest;
/**
 * Create a server for handling AWS Lambda requests.
 */
function createHandler(app, options = {}) {
    return function (event, context) {
        const { httpMethod: method } = event;
        const url = url_1.format({
            pathname: event.path,
            query: event.multiValueQueryStringParameters
        });
        const isBinary = options.isBinary || (() => false);
        const body = event.body
            ? Buffer.from(event.body, event.isBase64Encoded ? "base64" : "utf8")
            : undefined;
        const req = new LambdaRequest(url, {
            headers: event.multiValueHeaders,
            omitDefaultHeaders: true,
            method,
            body,
            context,
            event
        });
        const mapError = servie_errorhandler_1.errorhandler(req, {
            log: options.logError,
            production: options.production
        });
        function sendResponse(res) {
            req.signal.emit("responseStarted");
            return res.buffer().then(buffer => {
                const { status: statusCode } = res;
                const multiValueHeaders = toMultiValueHeaders(res.headers);
                const isBase64Encoded = isBinary(res);
                const body = buffer.toString(isBase64Encoded ? "base64" : "utf8");
                // Emit stats at end of response.
                req.signal.emit("responseBytes", buffer ? buffer.byteLength : 0);
                req.signal.emit("responseEnded");
                return {
                    statusCode,
                    multiValueHeaders,
                    body,
                    isBase64Encoded
                };
            }, err => sendResponse(mapError(err)));
        }
        // Marked request as finished.
        req.signal.emit("requestStarted");
        req.signal.emit("requestBytes", body ? body.byteLength : 0);
        req.signal.emit("requestStarted");
        return new Promise(resolve => {
            let result;
            req.signal.on("abort", () => {
                result = sendResponse(new node_1.Response(null, { status: 444 }));
                return resolve(result);
            });
            return resolve(Promise.resolve(app(req, servie_finalhandler_1.finalhandler(req))).then(res => result || sendResponse(res), err => result || sendResponse(mapError(err))));
        });
    };
}
exports.createHandler = createHandler;
/**
 * Return a lambda compatible object of headers.
 */
function toMultiValueHeaders(headers) {
    const result = Object.create(null);
    for (const key of headers.keys())
        result[key] = headers.getAll(key);
    return result;
}
//# sourceMappingURL=index.js.map